use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.002004
eval "use Test::Spelling 0.12; use Pod::Wordlist::hanekomu; 1" or die $@;

add_stopwords(<DATA>);
all_pod_files_spelling_ok('bin', 'lib');
__DATA__
Ioan
Rogers
lib
Net
LCDproc
Widget
Icon
String
HBar
Frame
VBar
Error
Scroller
Screen
Num
Meta
Attribute
Title
